==> 1.0.4.tar.gz: lib/src/observable.dart <==
import 'dart:collection';

import 'package:flutter/foundation.dart';

//_ObservableList that can retain a lot of callbacks.
class _ObservableList<T> extends Iterable<T> {
  final _map = LinkedHashMap<T, int>();

  _ObservableList();

  void add(T item) {
    if (item == null) return;
    _map[item] = 0;
  }

  bool remove(T item) {
    final int value = _map[item];
    if (value == null) {
      return false;
    }
    _map.remove(item);
    return true;
  }

  void clear() => _map.clear();

  @override
  bool contains(Object element) => _map.containsKey(element);

  @override
  Iterator<T> get iterator => _map.keys.iterator;

  @override
  bool get isEmpty => _map.isEmpty;

  @override
  bool get isNotEmpty => _map.isNotEmpty;

  factory _ObservableList.fromList(List<T> list) {
    final _observableList = _ObservableList<T>();
    for (var data in list) {
      _observableList.add(data);
    }
    return _observableList;
  }
}

//Callback interface to dispatch notification when a value is changed
typedef _ObservableCallback<T> = void Function(T event);

//Transformer interface that validate the value when it's changed and retain the error in an error object
typedef _ObservableTransformer<T, E> = E Function(T event);

//Observable class that manage a value and dispatch notifications when the value is change or validating;
class Observable<T, E> {
  final _listeners = _ObservableList<_ObservableCallback<T>>();
  T _value;
  E _error;
  _ObservableTransformer<T, E> _transformer;

  Observable([this._value]);

  T get value => _value;

  E get error => _error;

  _ObservableTransformer<T, E> get transformer => _transformer ?? (_) => null;

  @protected
  void notifyListeners() {
    for (var listener in _listeners) {
      _notifyListener(listener);
    }
  }

  void _notifyListener(_ObservableCallback<T> listener) {
    try {
      listener(_value);
    } catch (exception, stack) {
      FlutterError.reportError(
        FlutterErrorDetails(
          exception: exception,
          stack: stack,
          library: 'observable_pattern library',
          context: ErrorDescription(
              'while dispatching notifications for $runtimeType'),
          informationCollector: () sync* {
            yield DiagnosticsProperty<Observable<T, E>>(
              'The $runtimeType sending notification was',
              this,
              style: DiagnosticsTreeStyle.errorProperty,
            );
          },
        ),
      );
    }
  }

  set value(T value) {
    _value = value;
    _error = transformer(value);
    notifyListeners();
  }

  void add(T value) {
    _value = value;
    _error = transformer(value);
    notifyListeners();
  }

  set transformer(_ObservableTransformer<T, E> transformer) =>
      _transformer = transformer;

  void addListener(_ObservableCallback<T> callback) => _listeners.add(callback);

  void removeListener(_ObservableCallback<T> callback) =>
      _listeners.remove(callback);

  void validate() {
    _error = transformer(_value);
    notifyListeners();
  }

  bool get hasError => transformer(_value) != null;

  static Observable<bool, void> combine<T, E>(List<Observable<T, E>> list) {
    final observable = Observable<bool, void>(false);
    final observables = _ObservableList<Observable<T, E>>.fromList(list);
    observable.addListener((_) {
      for (var data in observables) {
        if (data.hasError) {
          observable._value = false;
          return;
        }
      }
      observable._value = true;
    });
    observable.transformer = (_) {
      for (var data in observables) {
        if (data.hasError) {
          data.validate();
        }
      }
      return null;
    };
    for (var data in observables) {
      data.addListener((_) {
        observable.notifyListeners();
      });
    }
    return observable;
  }
}

==> 1.0.5.tar.gz: lib/src/observable.dart <==
import 'dart:collection';

import 'package:flutter/foundation.dart';

///A class that can retain a lot of callbacks.
class _ObservableList<T> extends Iterable<T> {
  final _map = LinkedHashMap<T, int>();

  _ObservableList();

  void add(T item) {
    if (item == null) return;
    _map[item] = 0;
  }

  bool remove(T item) {
    final int value = _map[item];
    if (value == null) {
      return false;
    }
    _map.remove(item);
    return true;
  }

  void clear() => _map.clear();

  @override
  bool contains(Object element) => _map.containsKey(element);

  @override
  Iterator<T> get iterator => _map.keys.iterator;

  @override
  bool get isEmpty => _map.isEmpty;

  @override
  bool get isNotEmpty => _map.isNotEmpty;

  factory _ObservableList.fromList(List<T> list) {
    final _observableList = _ObservableList<T>();
    for (var data in list) {
      _observableList.add(data);
    }
    return _observableList;
  }
}

///An interface to dispatch notification when a value is changed
typedef _ObservableCallback<T> = void Function(T event);

///An interface that validate the value when it's changed and retain the error in an error object
typedef _ObservableTransformer<T, E> = E Function(T event);

///A class that manage a value and dispatch notifications when the value is change or validating;
class Observable<T, E> {
  final _listeners = _ObservableList<_ObservableCallback<T>>();
  bool _dirty = false;
  T _value;
  E _error;
  _ObservableTransformer<T, E> _transformer;

  Observable([this._value]);

  T get value => _value;

  E get error => _error;

  _ObservableTransformer<T, E> get transformer => _transformer ?? (_) => null;

  @protected
  void notifyListeners() {
    if (_dirty) {
      return;
    }
    _dirty = true;
    for (var listener in _listeners) {
      _notifyListener(listener);
    }
    _dirty = false;
  }

  void _notifyListener(_ObservableCallback<T> listener) {
    try {
      listener(_value);
    } catch (exception, stack) {
      FlutterError.reportError(
        FlutterErrorDetails(
          exception: exception,
          stack: stack,
          library: 'observable_pattern library',
          context: ErrorDescription(
              'while dispatching notifications for $runtimeType'),
          informationCollector: () sync* {
            yield DiagnosticsProperty<Observable<T, E>>(
              'The $runtimeType sending notification was',
              this,
              style: DiagnosticsTreeStyle.errorProperty,
            );
          },
        ),
      );
    }
  }

  set value(T value) {
    _value = value;
    _error = transformer(value);
    notifyListeners();
  }

  void add(T value) {
    _value = value;
    _error = transformer(value);
    notifyListeners();
  }

  set transformer(_ObservableTransformer<T, E> transformer) =>
      _transformer = transformer;

  void addListener(_ObservableCallback<T> callback) => _listeners.add(callback);

  void removeListener(_ObservableCallback<T> callback) =>
      _listeners.remove(callback);

  void validate() {
    _dirty = true;
    _error = transformer(_value);
    _dirty = false;
    notifyListeners();
  }

  bool get hasError => transformer(_value) != null;

  static Observable<bool, void> combine<T, E>(List<Observable<T, E>> list) {
    final observable = Observable<bool, void>(false);
    final observables = _ObservableList<Observable<T, E>>.fromList(list);
    observable.addListener((_) {
      for (var data in observables) {
        if (data.hasError) {
          observable._value = false;
          return;
        }
      }
      observable._value = true;
    });
    observable.transformer = (_) {
      for (var data in observables) {
        if (data.hasError) {
          data.validate();
        }
      }
      return null;
    };
    for (var data in observables) {
      data.addListener((_) {
        observable.notifyListeners();
      });
    }
    return observable;
  }
}
